import os
from threading import Thread

from prettytable import PrettyTable

from .cf_utils import read_data_from_file, write_data_to_file, Obj, obj_to_dict
from .codeforces import CodeforcesAPI

codeforces = CodeforcesAPI()
cache_loc = os.path.join(os.environ['HOME'], '.cache', 'cf_submit')
contests_loc = os.path.join(cache_loc, 'contests.json')


def load_contests(pretty_off):
    contests = [Obj(contest) for contest in read_data_from_file(contests_loc) or []]
    if len(contests) == 0:
        write_data_to_file(obj_to_dict(codeforces.contestList()), contests_loc)
        contests = [Obj(contest) for contest in read_data_from_file(contests_loc) or []]
    else:
        Thread(target=write_data_to_file, args=(codeforces.contestList(), contests_loc)).start()
    contests.sort(key=lambda contest: contest.id, reverse=True)
    if pretty_off:
        print(' '.join(map(str, map(lambda x: x.id, contests))))
    else:
        print_pretty(contests[0:20])


def print_pretty(contests):
    contests_table = PrettyTable()
    contests_table.field_names = ['Id', 'Name']
    for contest in contests:
        contests_table.add_row([contest.id, contest.name])
    contests_table.hrules = True
    contests_table.align['Name'] = 'l'
    print(contests_table.get_string(sortby='Id', reversesort=True))



import getpass
import os
import random

from robobrowser import RoboBrowser

from .cf_utils import write_data_to_file, read_data_from_file, Obj

root = '7'
cache_loc = os.path.join(os.environ['HOME'], '.cache', 'cf_submit')
config_loc = os.path.join(cache_loc, 'config.json')


# converter
def decode(s):
    global root
    res = ''
    length = len(s)
    i = 0
    while i < length:
        rng = ord(s[i]) - ord(root)
        jump = ord(s[i + 1]) - ord(root)
        temp = 0
        for j in range(0, rng):
            temp += ord(s[i + j + 2]) - ord(root) - jump
        res += str(chr(temp))
        i += rng + 2
    return res


def encode(s):
    global root
    res = ''
    length = len(s)
    for i in range(0, length):
        rng = random.randint(1, 20)
        res += str(chr(rng + ord(root)))
        jump = random.randint(1, 10)
        res += str(chr(jump + ord(root)))
        curr = ord(s[i])
        for j in range(0, rng - 1):
            temp = random.randint(0, min(curr, 2 + int(curr / (rng - j))))
            res += str(chr(temp + ord(root) + jump))
            curr -= temp
        res += str(chr(curr + ord(root) + jump))
    return res


# set login
def set_login(handle=None):
    if handle is None:
        handle = input('Handle: ')
    password = getpass.getpass('Password: ')

    browser = RoboBrowser(parser='lxml')
    browser.open('http://codeforces.com/enter')
    enter_form = browser.get_form('enterForm')
    enter_form['handleOrEmail'] = handle
    enter_form['password'] = password
    browser.submit_form(enter_form)

    checks = list(map(lambda x: x.getText()[1:].strip(),
                      browser.select('div.caption.titled')))
    if handle not in checks:
        print('Login Failed.')
        return
    else:
        config = read_data_from_file(config_loc)
        config['handle'] = handle
        config['password'] = encode(password)
        write_data_to_file(config, config_loc)
        print('Successfully logged in as ' + handle)


# login
def login():
    config = Obj(read_data_from_file(config_loc))
    handle = config.handle
    password = decode(config.password or None)

    browser = RoboBrowser(parser='lxml')
    browser.open('http://codeforces.com/enter')
    enter_form = browser.get_form('enterForm')
    enter_form['handleOrEmail'] = handle
    enter_form['password'] = password
    browser.submit_form(enter_form)

    checks = list(map(lambda x: x.getText()[1:].strip(), browser.select('div.caption.titled')))
    if handle not in checks:
        print('Login Corrupted.')
        return None
    else:
        return browser



import os
import re

from prettytable import PrettyTable

from . import cf_login
from .cf_utils import Obj, read_data_from_file, write_data_to_file, obj_to_dict

cache_loc = os.path.join(os.environ['HOME'], '.cache', 'cf_submit')
groups_loc = os.path.join(cache_loc, 'groups.json')
config_loc = os.path.join(cache_loc, 'config.json')


def refresh_contests_data(group):
    browser = cf_login.login()
    browser.open('https://codeforces.com/group/%s/contests' % group)
    raw_html = browser.parsed
    rows = raw_html.find('div', class_='datatable').find(
        'table').find_all('tr')[1:]
    contests = []
    for row in rows:
        contest = Obj({})
        m = re.search('data-contestid=".*"', str(row))
        setattr(contest, 'id', m.group(0).replace('data-contestid="', '')[:-1])
        setattr(contest, 'name', row.find_all('td')[0].text.split('\n')[1].strip())
        contests.append(contest)
    return contests


def refresh_groups_data():
    browser = cf_login.login()
    config = read_data_from_file(config_loc)
    browser.open('https://codeforces.com/groups/with/%s' % (config.get('handle', None)))
    raw_html = browser.parsed
    rows = raw_html.find('div', class_='datatable').find('table').find_all('tr')[1:]
    groups = []
    for row in rows:
        if str(row.find_all('td')[2].text).strip() != 'Accepted':
            continue
        group = Obj({})
        setattr(group, 'id',
                re.sub(r'.*/group/', '', re.sub(r'/members.*', '', str(row.find('a', class_='groupName')))))
        setattr(group, 'name', str(row.find('a', class_='groupName').text).strip())
        groups.append(group)
    write_data_to_file(obj_to_dict(groups), groups_loc)


def load_contests(group_id, pretty_off):
    groups = [Obj(group) for group in read_data_from_file(groups_loc) or []]
    if len(groups) == 0:
        refresh_groups_data()
        groups = [Obj(group) for group in read_data_from_file(groups_loc) or []]
    group = next((group for group in groups if group.id == group_id), None)
    if group is None:
        return
    if not hasattr(group, 'contests'):
        setattr(group, 'contests', refresh_contests_data(group_id))
        write_data_to_file(obj_to_dict(groups), groups_loc)

    if pretty_off:
        print(' '.join(map(str, map(lambda contest: contest.id, group.contests))))
    else:
        print_pretty(group.contests)


def load_groups(pretty_off):
    groups = [Obj(group) for group in read_data_from_file(groups_loc) or []]
    if len(groups) == 0:
        refresh_groups_data()
        groups = [Obj(group) for group in read_data_from_file(groups_loc) or []]
    if pretty_off:
        print(' '.join(map(str, map(lambda group: group.id, groups))))
    else:
        print_pretty([group for group in groups])


def print_pretty(data):
    contests = PrettyTable()
    contests.field_names = ['Id', 'Name']
    for i in data:
        contests.add_row([i.id, i.name])
    contests.hrules = True
    contests.align['Name'] = 'l'
    print(contests.get_string(sortby='Id'))



import os
from threading import Thread

from prettytable import PrettyTable

from .cf_utils import read_data_from_file, write_data_to_file, Obj, obj_to_dict
from .codeforces import CodeforcesAPI

codeforces = CodeforcesAPI()
cache_loc = os.path.join(os.environ['HOME'], '.cache', 'cf_submit')
gyms_loc = os.path.join(cache_loc, 'gyms.json')


def load_gyms(pretty_off):
    gyms = [Obj(gym) for gym in read_data_from_file(gyms_loc) or []]
    if len(gyms) == 0:
        write_data_to_file(obj_to_dict(codeforces.gymList()), gyms_loc)
        gyms = [Obj(gym) for gym in read_data_from_file(gyms_loc) or []]
    else:
        Thread(target=write_data_to_file, args=(codeforces.gymList(), gyms_loc)).start()
    gyms.sort(key=lambda gym: gym.id, reverse=True)
    if pretty_off:
        print(' '.join(map(str, map(lambda x: x.id, gyms))))
    else:
        print_pretty(gyms[0:20])


def print_pretty(gyms):
    gyms_table = PrettyTable()
    gyms_table.field_names = ['Id', 'Name']
    for gym in gyms:
        gyms_table.add_row([gym.id, gym.name])
    gyms_table.hrules = True
    gyms_table.align['Name'] = 'l'
    print(gyms_table.get_string(sortby='Id', reversesort=True))



import os
from subprocess import Popen

from . import cf_login
from .cf_colors import Colors


def parse(group, contest, problem):
    Popen(['rm', '-rf', 'files']).wait()
    Popen(['mkdir', '-p', 'files']).wait()
    print('%sImporting samples of problem %s...%s' % (Colors.WARNING, str(contest + problem), Colors.END))
    j = 0
    try:
        browser = cf_login.login()
        if group is not None:
            url = 'https://codeforces.com/group/%s/contest/%s/problem/%s' % (group, contest, problem)
        elif len(str(contest)) >= 6:
            url = 'https://codeforces.com/gym/%s/problem/%s' % (contest, problem)
        else:
            url = 'https://codeforces.com/contest/%s/problem/%s' % (contest, problem)
        browser.open(url)
        page = browser.parsed
        sample_test = page.find('div', class_='sample-test')
        inputs = sample_test.find_all('div', class_='input')
        for i in inputs:
            input_text = i.find('pre')
            input_text = str(input_text).replace(
                '<br/>', '\n').replace('<pre>', '').replace('</pre>', '')
            create_file(input_text.strip(), 'test%d.in' % j)
            j += 1
        outputs = sample_test.find_all('div', class_='output')
        j = 0
        for i in outputs:
            output_text = i.find('pre')
            output_text = str(output_text).replace(
                '<br/>', '\n').replace('<pre>', '').replace('</pre>', '')
            create_file(output_text.strip(), 'test%d.ans' % j)
            j += 1
    except Exception:
        print('%sError, try in few minutes!!%s' % (Colors.FAIL, Colors.END))
        return
    print('%s%d samples imported successfully!%s' % (Colors.OK_GREEN, j, Colors.END))


def create_file(source, file_name):
    file = open(os.path.join('files', file_name), 'w')
    file.write(source)
    file.close()



import os

from prettytable import PrettyTable

from . import cf_login
from . import cf_utils

cache_loc = os.path.join(os.environ['HOME'], '.cache', 'cf_submit')
config_loc = os.path.join(cache_loc, 'config.json')
problems_loc = os.path.join(cache_loc, 'problems.json')
config = cf_utils.read_data_from_file(config_loc) or {}
contest = config.get('contest', None)
group = config.get('group', None)


def refresh_problems_data():
    browser = cf_login.login()
    if group is not None:
        url = 'http://codeforces.com/group/' + group + '/contest/' + contest
    elif len(str(contest)) >= 6:
        url = 'http://codeforces.com/gym/' + contest
    else:
        url = 'http://codeforces.com/contest/' + contest
    browser.open(url)
    raw_html = browser.parsed
    data = []
    prob_rows = raw_html.find_all('table', class_='problems')[0].find_all('tr')
    for row in prob_rows[1:]:
        cell = row.find_all('td')
        if len(cell) < 4:
            continue
        problem = {
            'id': str(cell[0].get_text(strip=True)),
            'name': str(cell[1].find('a').get_text(strip=True))
        }
        nbr_solves = str(cell[3].get_text(strip=True))
        if len(nbr_solves) == 0:
            problem['solves'] = int(0)
        else:
            problem['solves'] = int(nbr_solves[1:])
        data.append(problem)
    return data


def load_problems(pretty_off):
    if contest is None:
        print('Set contest first with: cf con --id 1111')
        return

    problems = cf_utils.read_data_from_file(problems_loc) or {}
    if problems.get(contest, None) is None:
        problems[contest] = refresh_problems_data()
        cf_utils.write_data_to_file(problems, problems_loc)
    # printing
    if pretty_off:
        ids = [str(problem['id']).lower() for problem in problems[contest]]
        print(' '.join(map(str, ids)))
    else:
        print_pretty(problems[contest])


def print_pretty(data):
    problems = PrettyTable()
    problems.field_names = ['Id', 'Name', 'Solves']
    for i in data:
        problems.add_row([i['id'], i['name'], i['solves']])
    problems.align['Name'] = 'l'
    problems.align['Solves'] = 'r'
    print(problems.get_string(sortby='Id'))



import hashlib
import os
import time

import requests

from .cf_utils import Obj, read_data_from_file, random_digits_string

cache_loc = os.path.join(os.environ['HOME'], '.cache', 'cf_submit')
config_loc = os.path.join(cache_loc, 'config.json')


def generateAPISig(method, params):
    config = Obj(read_data_from_file(config_loc))
    if hasattr(config, 'api') and hasattr(config.api, 'key') and hasattr(config.api, 'secret'):
        api_key = config.api.key
        api_secret = config.api.secret
        unix_time = int(time.time())
        params.update({
            'apiKey': api_key,
            'time': unix_time,
        })
        r_string = random_digits_string(6)
        params_list = sorted([(key, params[key])
                              for key in params], key=lambda x: (x[0], x[1]))
        params_string = '&'.join(map(lambda x: '%s=%s' % (x[0], x[1]), params_list))
        to_be_encrypted = '%s%s?%s#%s' % (
            r_string, method, params_string, api_secret)
        return r_string + hashlib.sha512(to_be_encrypted.encode('utf-8')).hexdigest()
    else:
        return None


class CodeforcesAPI:
    baseUrl = 'https://codeforces.com/api'

    def contestList(self):
        response = requests.get('{}/contest.list'.format(self.baseUrl))
        return [Obj(res) for res in response.json()['result']]

    def contestStandings(self, contest_id, from_=1, count=10, show_unofficial=False):
        response = requests.get('{}/contest.standings'.format(self.baseUrl), params={
            'contestId': contest_id,
            'from': from_,
            'count': count,
            'showUnofficial': show_unofficial
        })
        return Obj(response.json()['result'])

    def contestHacks(self, contest_id):
        response = requests.get('{}/contest.hacks'.format(self.baseUrl), params={
            'contestId': contest_id
        })
        return [Obj(res) for res in response.json()['result']]

    def gymList(self):
        response = requests.get('{}/contest.list'.format(self.baseUrl), params={
            'gym': True
        })
        return [Obj(res) for res in response.json()['result']]

    def userSubmissions(self, handle, from_=1, count=100):
        params = {
            'handle': handle,
            'from': from_,
            'count': count,
        }
        api_sig = generateAPISig('/user.status', params)
        if api_sig is not None:
            params.update({'apiSig': api_sig})
        response = requests.get('{}/user.status'.format(self.baseUrl), params=params)
        return [Obj(res) for res in response.json()['result']]



import json
import os
import random
import string
from json import JSONDecodeError


class Obj(object):
    def __init__(self, d):
        for a, b in d.items():
            if isinstance(b, (list, tuple)):
                setattr(self, a, [Obj(x) if isinstance(
                    x, dict) else x for x in b])
            else:
                setattr(self, a, Obj(b) if isinstance(b, dict) else b)

    def toString(self):
        return json.dumps(self, default=lambda x: x.__dict__, indent=2)


def obj_to_dict(obj):
    if isinstance(obj, (list, tuple)):
        return [obj_to_dict(o) for o in obj]
    elif isinstance(obj, Obj):
        res = dict()
        for key, value in obj.__dict__.items():
            res[key] = obj_to_dict(value)
        return res
    else:
        return obj


def write_data_to_file(data, file_name):
    try:
        with open(file_name, 'w') as file:
            json.dump(data, file, indent=2)
            return True
    except (TypeError, OSError):
        return False


def read_data_from_file(file_name):
    try:
        if os.path.isfile(file_name):
            with open(file_name, 'r') as file:
                return json.load(file)
        else:
            return None
    except (JSONDecodeError, OSError):
        return None


def random_digits_string(length=10):
    return ''.join(random.choice(string.digits) for _ in range(length))


def safe_list_get(list_, index, default=None):
    try:
        return list_[index]
    except IndexError:
        return default



import re
import sys
import time

from .cf_colors import Colors
from .cf_utils import safe_list_get
from .codeforces import CodeforcesAPI

codeforces = CodeforcesAPI()


# submissions
def get_submission_data(handle):
    submission = safe_list_get(codeforces.userSubmissions(handle, count=1), 0)
    if submission is None:
        print('Connection Error!')
    # check if verdict exists (in queue if not)
    if not hasattr(submission, 'verdict'):
        setattr(submission, 'verdict', 'IN QUEUE')
    return submission


# look at last submission
def peek(handle):
    submission = get_submission_data(handle)
    if not hasattr(submission.problem, 'contestId'):
        setattr(submission.problem, 'contestId', 'guru')
    problem = '%s%s' % (submission.problem.contestId, submission.problem.index)
    if submission.verdict == 'TESTING':
        print('Submission %s to problem %s: %s'
              % (submission.id, problem, submission.verdict))
    else:
        if submission.verdict != 'OK':
            print('Submission %s to problem %s: %s on test %s\nTime:   %sms \nMemory: %sKb'
                  % (submission.id, problem, submission.verdict, 1 + submission.passedTestCount,
                     submission.timeConsumedMillis, submission.memoryConsumedBytes / 1024))
        else:
            print('Submission %s to problem %s: %s! passed all %s tests\nTime:   %sms \nMemory: %sKb'
                  % (submission.id, problem, submission.verdict, submission.passedTestCount,
                     submission.timeConsumedMillis, submission.memoryConsumedBytes / 1024))


# watch last submission
def watch(handle):
    spinner = {0: '◴', 1: '◵', 2: '◶', 3: '◷'}
    count = 0
    while True:
        submission = get_submission_data(handle)
        if not hasattr(submission.problem, 'contestId'):
            setattr(submission.problem, 'contestId', 'guru')
        problem = '%s%s' % (submission.problem.contestId,
                            submission.problem.index)
        if submission.verdict not in ['TESTING', 'IN QUEUE']:
            if submission.verdict != 'OK':
                sys.stdout.write('\rSubmission %s to problem %s: %s on test %s\nTime:   %sms \nMemory: %sKb'
                                 % (submission.id, problem, submission.verdict, 1 + submission.passedTestCount,
                                    submission.timeConsumedMillis, submission.memoryConsumedBytes / 1024))
            else:
                sys.stdout.write('\rSubmission %s to problem %s: %s! passed all %s tests\nTime:   %sms \nMemory: %sKb'
                                 % (submission.id, problem, submission.verdict, submission.passedTestCount,
                                    submission.timeConsumedMillis, submission.memoryConsumedBytes / 1024))
            sys.stdout.flush()
            break
        else:
            sys.stdout.write('\rSubmission %s to problem %s: %s ....... %s    '
                             % (submission.id, problem, submission.verdict, spinner[count]))
            sys.stdout.flush()
            count = (count + 1) % 4
        time.sleep(0.25)
    print('\a')


# submit problem

def submit_problem(browser, contest, lang, source, guru):
    # get form
    submission = browser.get_form(class_='submit-form')
    if submission is None:
        print('Cannot find problem')
        return False

    # submit form
    submission['sourceFile'] = source
    if lang == 'cpp':
        # GNU G++14 6.2.0
        lang_code = '50'
        # GNU G++11 5.1.0
        # lang_code = '42'
    elif lang == 'c':
        # GNU GCC C11 5.1.0
        lang_code = '43'
    elif lang == 'd':
        lang_code = '28'
    elif lang == 'py':
        # python 2.7.12
        # lang_code = '7'
        # python 3.5.2
        lang_code = '31'
    elif lang == 'rb':
        # Ruby 2.0.0p645
        lang_code = '8'
    elif lang == 'kt':
        lang_code = '48'
    elif lang == 'java':
        # Java 1.8.0_112
        lang_code = '36'
    elif lang == 'scala':
        lang_code = '20'
    elif lang == 'rs':
        lang_code = '49'
    elif lang == 'php':
        lang_code = '6'
    else:
        print('Unknown Language')
        return False
    submission['programTypeId'] = lang_code

    # check acmsguru
    if guru != -1:
        submission['submittedProblemCode'] = guru

    browser.submit_form(submission)

    # check if good
    if (guru != -1 and browser.url[-7:] != '/status') or (guru == -1 and browser.url[-3:] != '/my'):
        print('Failed to submit code')
        print(' @ %s' % (str(browser.url)))
        return False
    print('Code submitted properly')

    # now get time
    countdown_timer = browser.parsed.find_all(
        'span', class_='contest-state-regular countdown before-contest-' + contest + '-finish')
    if len(countdown_timer) > 0:
        print('%sTIME LEFT: %s%s' % (Colors.BOLD, str(countdown_timer[0].get_text(strip=True)), Colors.END))

    return True


# submit problem
def submit(browser, handle, group, contest, problem, lang, source, show, guru):
    if guru:
        print('Submitting to acmsguru %s as %s'
              % (problem, handle))
    elif group is not None:
        print('Submitting to problem %s%s in group %s as %s'
              % (contest, problem.upper(), group, handle))
    else:
        print('Submitting to problem %s%s as %s'
              % (contest, problem.upper(), handle))

    pid = -1
    if guru:
        browser.open('http://codeforces.com/problemsets/acmsguru/submit')
        pid = problem
    elif group is not None:
        browser.open('http://codeforces.com/group/%s/contest/%s/submit/%s'
                     % (group, contest, problem.upper()))
    elif len(contest) >= 6:
        browser.open('http://codeforces.com/gym/%s/submit/%s'
                     % (contest, problem.upper()))
    else:
        browser.open('http://codeforces.com/contest/%s/submit/%s'
                     % (contest, problem.upper()))

    # show submission
    if submit_problem(browser, contest, lang, source, pid) and show:
        watch(handle)


# submit, possibly len(args) > 1
def submit_files(browser, default_handle, default_group, default_contest, default_prob, def_ext, default_lang,
                 args, show, guru):
    # if len == 0, query for file
    if len(args) == 0:
        args.append(input('File to submit: '))

    for source in args:
        # split file name
        info = source.split('.')
        # single filename
        if source.find('.') == -1:
            info.append(def_ext)
            source += '.' + def_ext

        # check language
        if default_lang is not None:
            info[-1] = default_lang

        # submit problem
        if default_prob is not None:
            if len(default_prob) == 1:
                # letter only
                submit(browser, default_handle, default_group, default_contest,
                       default_prob, info[-1], source, show, guru)
            elif len(default_prob) == 2:
                # letter + number (?)
                submit(browser, default_handle, default_group, default_contest,
                       default_prob, info[-1], source, show, guru)
            else:
                #  parse string
                split = re.split(r'(\D+)', default_prob)
                if len(split) == 3 and len(split[1]) == 1 and len(split[2]) == 0:
                    # probably a good string
                    submit(browser, default_handle, default_group,
                           split[0], split[1], info[-1], source, show, guru)
                else:
                    print('cannot understand the problem specified')

        elif len(info) == 2:
            if guru:
                # ACMSGURU
                submit(browser, default_handle, default_group,
                       default_contest, info[0], info[1], source, show, guru)
            else:
                # CODEFORCES
                # try to parse info[0]
                if info[0][:2].lower() == 'cf':
                    # remove the cf
                    info[0] = info[0][2:]
                if len(info[0]) == 1:
                    # only the letter, use default contest
                    submit(browser, default_handle, default_group,
                           default_contest, info[0], info[1], source, show, guru)
                else:
                    # contest is included, so parse
                    split = re.split(r'(\D+)', info[0])
                    if len(split) == 3 and len(split[1]) == 1 and len(split[2]) == 0:
                        # probably good string ?
                        submit(browser, default_handle, default_group,
                               split[0], split[1], info[1], source, show, guru)
                    else:
                        print(
                            'cannot parse filename, specify problem with -p or --prob')
        else:
            print('cannot parse filename, specify problem with -p or --prob')



import re

from prettytable import PrettyTable

from . import cf_login
from .cf_colors import Colors


def makeAscii(s):
    return re.sub(r'[^\x00-\x7f]', r'?', s)


# print friends standings in specified contest
# parse html
# gym contest
def print_standings(group, contest, verbose, top, sort, show_all):
    # requires login
    browser = cf_login.login()
    if group is not None:
        # group contest contest
        url = 'http://codeforces.com/group/' + \
              group + '/contest/' + contest + '/standings'
    elif len(str(contest)) >= 6:
        # gym contest
        url = 'http://codeforces.com/gym/' + contest + '/standings'
    else:
        # codeforces round
        url = 'http://codeforces.com/contest/' + contest + '/standings'
    # check if friends
    if group is not None:
        url += '/groupmates/true'
    elif show_all is False:
        url += '/friends/true'
    else:
        url += '/page/1'
    browser.open(url)
    raw_html = browser.parsed
    # get standings table
    handle_dict = {}
    mellon = raw_html.find('table', class_='standings').find_all('tr')

    standings = PrettyTable()

    # get header
    first_part = len(mellon[0].find_all('th')) - len(mellon[0].find_all('a'))
    header = []
    if verbose:
        for h_cell in mellon[0].find_all('th')[:first_part]:
            h_cell_str = str(h_cell.get_text(strip=True))
            if h_cell_str == '*':
                h_cell_str = 'Hacks'
            header.append(h_cell_str)
    else:
        for h_cell in mellon[0].find_all('th')[1:3]:
            header.append(str(h_cell.get_text(strip=True)))
    for h_cell in mellon[0].find_all('a'):
        header.append(str(h_cell.get_text(strip=True)))
    # append sorter to header
    if sort is not None:
        header.append('sorter')
    standings.field_names = header

    id_start = 4
    if not verbose:
        id_start = 0
        for h_cell in mellon[0].find_all('th'):
            h_cell_str = str(h_cell.get_text(strip=True))
            if h_cell_str.find('A') != -1:
                break
            id_start += 1

    fields = dict()
    for i, h in enumerate(header):
        fields[h] = i

    # find problem start and solve column
    problem_start = 0
    while header[problem_start].find('A') == -1:
        problem_start += 1
    solve_col = 0
    while header[solve_col] != '=':
        solve_col += 1

    # if sort, use dict
    if sort is not None:
        handle_dict = {}

    # fix top
    top = min(top + 1, len(mellon) - 1)

    # iterate
    for ami in mellon[1:top]:
        virtual = True
        row = ami.find_all('td')
        table_row = []
        # get place
        # this cell has problems
        rank = str(row[0].get_text(strip=True))
        if len(rank) == 0:
            virtual = False
        if verbose:
            if rank.find(')') != -1:
                rank = rank[rank.find('(') + 1:rank.find(')')]
            table_row.append(rank)

        # get name
        party = str(row[1].get_text(strip=True))
        if verbose:
            team = []
            # check if virtual time colon
            virtual_time = None
            if party[-3] == ':':
                virtual_time = party[-5:]
                party = party[:-5]

            # check if there are still colons left if yes, split at last colon
            if party.count(':') > 0:
                # check for '#'
                tail = ''
                if party[-1] == '#':
                    tail = '#'
                    party = party[:-1]
                # split
                party = party.split(':')
                # get first part (team name)
                team_name = party[0]
                for party_part in party[1:-1]:
                    team_name += ':' + party_part
                if len(team_name + tail) > 24:
                    team_name = team_name[:20] + '...'
                team_name += tail + ':'
                team.append(team_name)
                # split rest of team members
                for member in party[-1].split(','):
                    team.append(member.strip())
            else:
                team.append(party)
            # append time if it exists'''
            if virtual_time is not None:
                team.append(virtual_time)
            # join party
            party = '\n'.join(team)
        else:
            if party[-3] == ':':
                party = party[:-5]
            if party.count(':') > 0:
                tail = ''
                if party[-1] == '#':
                    tail = '#'
                    party = party[:-1]
                # split
                party = party.split(':')
                team_name = party[0]
                for party_part in party[1:-1]:
                    team_name += ':' + party_part
                if len(team_name) > 32:
                    team_name = team_name[:32] + '...'
                team_name += tail
                party = team_name
        table_row.append(makeAscii(party))

        # get points or number of solves
        table_row.append(int(str(row[2].get_text(strip=True))))
        # get penalty or hacks
        if verbose:
            table_row.append(str(row[3].get_text(strip=True)))

        # get problem submissions
        for cell in row[id_start:]:
            problem_res = str(cell.get_text(strip=True))
            if virtual and len(problem_res) > 5:
                if verbose:
                    problem_res = (problem_res[:-5] + '\n' + problem_res[-5:])
                else:
                    problem_res = problem_res[:-5]
            else:
                problem_res = problem_res.replace('-', 'WA-')
            table_row.append(problem_res)

        # check sort
        if sort is not None:
            # add to dict
            # make party legible
            if party[0] == '*':
                party = party[1:]
            if party[-1] == '#':
                party = party[:-1]

            # check if party exists
            if party not in handle_dict:
                handle_dict[party] = table_row
            else:
                # update
                for i in range(problem_start, len(header) - 1):
                    # update if empty or wa
                    if len(handle_dict[party][i]) == 0:
                        handle_dict[party][i] = table_row[i]
                        # check if we should update solve_col
                        if len(table_row[i]) != 0 and table_row[i][0] == '+':
                            handle_dict[party][solve_col] += 1
                    elif handle_dict[party][i][0] != '+' and len(table_row[i]) != 0:
                        total_wa = int(handle_dict[party][i].split(
                            '\n')[0].split('-')[1])
                        if table_row[i].find('-') != -1:
                            # add wa
                            total_wa += int(table_row[i].split('\n')[0].split('-')[1])
                            handle_dict[party][i] = 'WA-' + str(total_wa)
                        else:
                            # add wa to correct submission
                            correct = table_row[i].split('\n')[0][1:]
                            if len(correct) != 0:
                                total_wa += int(correct)
                            handle_dict[party][i] = '+' + str(total_wa)
                            # update solve_col
                            handle_dict[party][solve_col] += 1
        else:
            # NO sort, add to table_row
            standings.add_row(table_row)

    # standings properties
    if verbose:
        standings.hrules = True
        if 'Penalty' in standings.align:
            standings.align['Penalty'] = 'r'
    standings.align['Who'] = 'l'
    standings.align['='] = 'r'

    # print standings
    if sort is None:
        print(standings)

    elif sort == 'solves':
        # add row_info to standings
        for _, row_info in handle_dict.items():
            # append the sorter
            sort_value = row_info[fields['=']]
            if 'Penalty' in fields:
                if len(row_info[fields['Penalty']]) == 0:
                    sort_value = 100000 * sort_value - 99999
                else:
                    sort_value = 100000 * sort_value - \
                                 int(row_info[fields['Penalty']])
            row_info.append(sort_value)
            standings.add_row(row_info)
        # print
        print(standings.get_string(
            sortby='sorter',
            reversesort=True,
            fields=header[:-1]
        ))

    elif sort == 'index':
        print('sort == index : nothing here')

    else:
        print('this should not have happened. nothing here')

    # first check if countdown
    # bold_start = '\033[1m'
    # bold_end = '\033[0;0m'
    countdown_timer = raw_html.find_all('span', class_='countdown')
    if len(countdown_timer) > 0:
        print('%sTIME LEFT: %s%s' % (Colors.BOLD, str(countdown_timer[0].get_text(strip=True)), Colors.END))


# get time
def print_time(group, contest):
    browser = cf_login.login()
    if contest is None:
        print('Set contest first with: cf con/gym --id 1111')
        return
    if group is not None:
        url = 'http://codeforces.com/group/' + group + '/contest/' + contest + '/submit'
    elif len(str(contest)) >= 6:
        url = 'http://codeforces.com/gym/' + contest + '/submit'
    else:
        url = 'http://codeforces.com/contest/' + contest + '/submit'
    browser.open(url)
    countdown_timer = browser.parsed.find_all(
        'span', class_='contest-state-regular countdown before-contest-' + contest + '-finish')
    if len(countdown_timer) == 0:
        print('Contest ' + contest + ' is over')
    else:
        print('%sTIME LEFT: %s%s' % (Colors.BOLD, str(countdown_timer[0].get_text(strip=True)), Colors.END))



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='cf_submit',
    version='1.3.5',
    scripts=['cf'],
    author='Nasreddine Bac Ali',
    author_email='nasreddine.bacali95@gmail.com',
    description='Submit Codeforces codes via terminal and other coll stuff',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/bacali95/cf_submit',
    packages=find_packages(),
    package_data={
        'cf_submit': [
            'bin/cf_checker',
            'bash_completion/cf'
        ]
    },
    install_requires=[
        'lxml',
        'robobrowser',
        'prettytable',
        'javalang',
        'requests',
        'Werkzeug>=0.16,<1.0'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ]
)



import os
import re
import time
from subprocess import Popen, TimeoutExpired

import javalang
from javalang.tree import ClassDeclaration
from prettytable import PrettyTable
from robobrowser import RoboBrowser

from . import cf_login
from .cf_colors import Colors
from .cf_utils import safe_list_get
from .codeforces import CodeforcesAPI

dir_path = os.getcwd()


# Hack problems
def begin_hack(contest, problem, generator, tle_generator, checker, correct_solution, max_tests, reverse, time_limit):
    # Preparing Workspace
    init_hack_dir(generator, tle_generator, checker, correct_solution)
    generator = 'workspace/%s' % generator
    checker = 'workspace/%s' % checker
    correct_solution = 'workspace/%s' % correct_solution
    if tle_generator is not None:
        tle_generator = 'workspace/%s' % tle_generator

    with open('tried_submissions', 'r') as file:
        list_str = file.read().strip()
        tried_submissions = list()
        if list_str != '':
            tried_submissions = list(map(int, list_str.split(' ')))
    tried_submissions_file = open('tried_submissions', 'a')

    tried_solutions = 0
    hacked_solutions = 0

    browser = get_browser_for_url(
        'https://codeforces.com/contest/%s/status/%s' % (contest, problem))
    max_pages = int(browser.parsed.find_all(class_='page-index')[-1].text)
    print('\n%sHappy Hacking 3:) - max pages : %s%s' %
          (Colors.HEADER, max_pages, Colors.END))
    start = max_pages if reverse else 1
    stop = 0 if reverse else max_pages + 1
    for i in range(start, stop, -1 if reverse else 1):
        try:
            browser = get_browser_for_url(
                'https://codeforces.com/contest/%s/status/%s/page/%s?order=BY_CONSUMED_TIME_ASC'
                % (contest, problem, i))
            submissions = browser.parsed.find_all(
                'table', class_='status-frame-datatable')[0].find_all('tr')[1:]
            for submission in submissions:
                try:
                    submission_id = int(submission.find(
                        'td', class_='id-cell').find('a').text)
                    if submission_id in tried_submissions:
                        print('\n%sSubmission %d on page %d/%d already tried!!%s' %
                              (Colors.WARNING, submission_id, i, max_pages, Colors.END))
                        continue
                    language = submission.find_all(
                        'td')[4].text.strip().replace(' ', '')
                    browser = get_browser_for_url(
                        'http://codeforces.com/contest/%s/submission/%d' % (contest, submission_id))
                    if len(browser.parsed.find_all('pre', class_='program-source')) > 0:
                        Popen(['rm', '-rf', 'workspace/testing_dir']).wait()
                        Popen(['mkdir', '-p', 'workspace/testing_dir']).wait()

                        source = browser.parsed.find_all(
                            'pre', class_='program-source')[0].text
                        player_solution = create_player_solution_file(
                            source, get_language_key(language))
                        if player_solution is None:
                            continue

                        print('\n%sHacked : %d, %sFailed : %d, %sTotal : %d%s'
                              % (Colors.OK_GREEN, hacked_solutions, Colors.FAIL, tried_solutions - hacked_solutions,
                                 Colors.OK_BLUE, tried_solutions, Colors.END))
                        print('%sTrying to hack a %s solution - %d on page %d/%d...%s'
                              % (Colors.HEADER, language, submission_id, i, max_pages, Colors.END))
                        print('%sNormal hack process%s' %
                              (Colors.WARNING, Colors.END))

                        comp(player_solution)
                        language = get_language_key(language)
                        result = execute_hack_process(generator, checker, correct_solution, player_solution,
                                                      language, time_limit, max_tests)
                        if result == 'CHECKER_ERROR':
                            test_hack_loc = os.path.join(
                                dir_path, 'workspace/testing_dir/test.in')
                            print('%sHope that will win 3:)%s' %
                                  (Colors.OK_GREEN, Colors.END))
                            submit_hack(contest, test_hack_loc, submission_id)
                            hacked_solutions = hacked_solutions + 1
                            tried_submissions_file.write(' %s' % submission_id)
                            tried_submissions_file.flush()
                            continue

                        if tle_generator is not None:
                            print(
                                '%sTLE hack process with time limit %ss %s' % (Colors.WARNING, time_limit, Colors.END))
                            result = execute_hack_process(tle_generator, checker, correct_solution, player_solution,
                                                          language, time_limit)
                            if result in ['CHECKER_ERROR', 'TIME_LIMIT_EXCEEDED']:
                                test_hack_loc = os.path.join(
                                    dir_path, tle_generator)
                                print('%sHope that will win 3:)%s' %
                                      (Colors.OK_GREEN, Colors.END))
                                submit_tle_hack(
                                    contest, test_hack_loc, submission_id)
                                hacked_solutions = hacked_solutions + 1
                                tried_submissions_file.write(
                                    ' %s' % submission_id)
                                tried_submissions_file.flush()
                                continue

                        tried_solutions = tried_solutions + 1
                        tried_submissions_file.write(' %s' % submission_id)
                        tried_submissions_file.flush()
                except KeyboardInterrupt:
                    time.sleep(2)
                    break
                except Exception:
                    continue
        except KeyboardInterrupt:
            time.sleep(2)
            break
        except Exception:
            continue
    print('\n%sRESULT => %sHacked : %d, %sFailed : %d, %sTotal : %d'
          % (Colors.HEADER, Colors.OK_GREEN, hacked_solutions, Colors.FAIL,
             tried_solutions - hacked_solutions, Colors.OK_BLUE, tried_solutions))


def submit_hack(contest, hack_test, submission_id):
    browser = cf_login.login()
    browser.open('https://codeforces.com/contest/' +
                 contest + '/challenge/' + str(submission_id))
    hack_form = browser.get_form(class_='challenge-form')
    hack_form['testcaseFromFile'] = hack_test
    browser.submit_form(hack_form)


def submit_tle_hack(contest, hack_generator, submission_id):
    browser = cf_login.login()
    browser.open('https://codeforces.com/contest/' +
                 contest + '/challenge/' + str(submission_id))
    hack_form = browser.get_form(class_='challenge-form')
    hack_form['generatorSourceFile'] = hack_generator
    hack_form['programTypeId'] = '50'
    browser.submit_form(hack_form)


def comp(source):
    info = source.split('.')
    lang = info[-1]
    f_null = open(os.devnull, 'w')
    if lang == 'cpp':
        Popen('g++ %s -DONLINE_JUDGE -o %s' % (source, info[0]), stdout=f_null, stderr=f_null, shell=True).wait()
    elif lang == 'c':
        Popen('gcc %s -DONLINE_JUDGE -o %s' % (source, info[0]), stdout=f_null, stderr=f_null, shell=True).wait()
    elif lang == 'java':
        Popen('javac %s' % source, stdout=f_null, stderr=f_null, shell=True).wait()
    elif lang == 'kt':
        Popen('kotlinc %s -include-runtime -d %s' % (source, info[0] + '.jar'), stdout=f_null, stderr=f_null,
              shell=True).wait()


def execute(source, args=None, language=None, input_file=None, output_file=None, timeout=10):
    if args is None:
        args = []
    executable = source.split('.')[0]
    if language is None:
        language = source.split('.')[-1]
        language = 'py3' if language == 'py' else language
    if language in ['cpp', 'c']:
        cmd = './%s ' % executable
    elif language == 'java':
        cmd = 'java -DONLINE_JUDGE %s ' % executable
    elif language == 'kt':
        cmd = 'java -DONLINE_JUDGE -jar %s.jar ' % executable
    elif language == 'py2':
        cmd = 'python2 %s ' % source
    elif language == 'py3':
        cmd = 'python3 %s ' % source
    else:
        print('Sorry language not supported!')
        return exit(-1)
    return_code = -1
    process = None
    try:
        process = Popen(cmd + ' '.join(list(map(str, args))), stdin=input_file, stdout=output_file, shell=True)
        process.wait(int(timeout))
        return_code = process.returncode
    except TimeoutExpired:
        process.kill()
        return_code = 98989898
    except Exception:
        return_code = -1
    finally:
        return return_code


def execute_hack_process(generator, checker, correct_solution, player_solution, player_language, time_limit,
                         max_tests=1):
    for test_index in range(0, max_tests):
        input_file = open('workspace/testing_dir/test.in', 'w')
        return_code = execute(generator, args=[test_index], output_file=input_file)
        if return_code != 0:
            return 'GENERATOR_ERROR'

        input_file = open('workspace/testing_dir/test.in', 'r')
        answer_file = open('workspace/testing_dir/test.ans', 'w')
        return_code = execute(correct_solution, input_file=input_file, output_file=answer_file)
        if return_code != 0:
            return 'CORRECT_SOLUTION_ERROR'

        output_file = open('workspace/testing_dir/test.out', 'w')
        return_code = execute(player_solution, language=player_language, args=['<', 'workspace/testing_dir/test.in'],
                              output_file=output_file, timeout=time_limit)

        if return_code == 98989898:
            return 'TIME_LIMIT_EXCEEDED'

        if return_code != 0:
            return 'PLAYER_SOLUTION_ERROR'

        return_code = execute(checker, args=['workspace/testing_dir/test.in', 'workspace/testing_dir/test.out',
                                             'workspace/testing_dir/test.ans'])
        if return_code != 0:
            return 'CHECKER_ERROR'
    return 'ALL_TESTS_PASSED'


def init_hack_dir(generator, tle_generator, checker, correct_solution):
    print('%sInitializing workspace...%s' % (Colors.OK_GREEN, Colors.END))
    Popen(['touch', 'tried_submissions']).wait()
    Popen(['rm', '-rf', 'workspace']).wait()
    Popen(['mkdir', '-p', 'workspace']).wait()
    Popen(['cp', 'testlib.h', 'workspace']).wait()
    Popen(['cp', generator, 'workspace']).wait()
    Popen(['cp', checker, 'workspace']).wait()
    Popen(['cp', correct_solution, 'workspace']).wait()

    comp('workspace/%s' % generator)
    comp('workspace/%s' % checker)
    comp('workspace/%s' % correct_solution)
    if tle_generator is not None:
        Popen(['cp', tle_generator, 'workspace']).wait()
        comp('workspace/%s' % tle_generator)

    print('%sWorkspace is ready!!%s' % (Colors.OK_GREEN, Colors.END))


def get_browser_for_url(url):
    browser = RoboBrowser(parser='lxml')
    browser.open(url)
    return browser


def get_language_key(language):
    if re.match(r'(.)*\+\+(.)*', language):
        return 'cpp'
    elif re.match(r'(.)*GNU(.)*', language):
        return 'c'
    elif re.match(r'(.)*Kotlin(.)*', language):
        return 'c'
    elif re.match(r'(.)*Java(.)*', language):
        return 'java'
    elif re.match(r'(.)*Kotlin(.)*', language):
        return 'kt'
    elif re.match(r'(.)*Py(.)*2', language):
        return 'py2'
    elif re.match(r'(.)*Py(.)*3', language):
        return 'py3'
    else:
        return None


def create_player_solution_file(source, language):
    if language == 'cpp':
        file_name = 'noncorrect.cpp'
    elif language == 'c':
        file_name = 'noncorrect.c'
    elif language == 'kt':
        file_name = 'noncorrect.kt'
        source = 'package workspace.testing_dir;\n' + source
    elif language == 'java':
        try:
            tree = javalang.parse.parse(source)
            name = next(klass.name for klass in tree.types
                        if isinstance(klass, ClassDeclaration)
                        for m in klass.methods
                        if m.name == 'main' and m.modifiers.issuperset({'public', 'static'}))
            file_name = name + '.java'
            source = 'package workspace.testing_dir;\n' + source
        except TypeError:
            return None
    elif language in ['py2', 'py3']:
        file_name = 'noncorrect.py'
    else:
        return None
    file_name = 'workspace/testing_dir/%s' % file_name
    for_hack_source = open(file_name, 'w')
    for_hack_source.write(source)
    for_hack_source.close()
    return file_name


def print_standings(contest, limit, show_all):
    if contest is None:
        print('Please specify a contest first using: cf hack standings --contest 1010 or cf con --id 1010')
        return
    codeforces = CodeforcesAPI()
    standings = codeforces.contestStandings(
        contest, count=None, show_unofficial=True)
    rows = standings.rows
    print('Contest:\n   Id:   %s\n   Name: %s' %
          (contest, standings.contest.name))
    data = list()
    standings = PrettyTable()
    standings.field_names = ['Rank', 'Name', 'Hacks']
    standings.align['Name'] = 'l'
    standings.align['Hacks'] = 'l'
    for row in rows:
        if row.party.ghost is True:
            handle = row.party.teamName
        else:
            handle = safe_list_get(row.party.members, 0, {'handle': None}).handle
        entry = {
            'handle': handle,
            'successfulHackCount': row.successfulHackCount,
            'unsuccessfulHackCount': row.unsuccessfulHackCount
        }
        data.append(entry)

    data.sort(key=lambda elem: elem['unsuccessfulHackCount'])
    data.sort(key=lambda elem: elem['successfulHackCount'], reverse=True)
    for i, item in enumerate(data):
        if i >= limit and not show_all:
            break
        standings.add_row(
            [i + 1, item['handle'], '+{} : -{}'.format(
                item['successfulHackCount'], item['unsuccessfulHackCount']
            )]
        )
    print(standings.get_string(sortby='Rank'))



import argparse
import os
import re
import webbrowser

from . import __version__
from . import cf_contests
from . import cf_groups
from . import cf_gyms
from . import cf_hack
from . import cf_login
from . import cf_parse
from . import cf_problems
from . import cf_standings
from . import cf_submit
from . import cf_test
from . import cf_utils


# main
def main():
    cache_loc = os.path.join(os.environ['HOME'], '.cache', 'cf_submit')
    if os.path.isdir(cache_loc) is False:
        os.mkdir(cache_loc)
    config_loc = os.path.join(cache_loc, 'config.json')
    config = cf_utils.read_data_from_file(config_loc)
    if config is None:
        config = dict()
        cf_utils.write_data_to_file(config, config_loc)

    # ------------------- argParse --------------------
    parser = argparse.ArgumentParser(
        description='Command line tool to submit to codeforces', formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('command',
                        help='con/gym/gcon -- change contest or gym or group contest id\n' +
                             'hack -- try to hack all the accepted submissions for a specific given problem\n' +
                             'ext -- change default file extension\n' +
                             'ext -- change default file extension\n' +
                             'info -- current handle and contest id\n' +
                             'login -- save login info\n' +
                             'peek -- look at last submission\n' +
                             'problems -- show number of solves on each problem\n' +
                             'standings -- show standings of friends in default contest, or specify contest with -p\n' +
                             'submit -- submit code to problem\n' +
                             'time -- shows time left in contest\n' +
                             'watch -- watch last submission\n' +
                             'open -- open selected problem on default browser\n' +
                             'parse -- import selected problem samples data\n' +
                             'test -- test the selected source code with the imported tests data\n')

    parser.add_argument('option',
                        nargs='*', default=None,
                        help='file to submit')

    parser.add_argument('--handle',
                        action='store', default=None,
                        help='specify handle, example: --handle _bacali')

    parser.add_argument('--id',
                        action='store', default=None,
                        help='specify contest or gym id, example: --id 1117')

    parser.add_argument('-p', '--prob',
                        action='store', default=None,
                        help='specify problem, example: -p 845a')

    parser.add_argument('--pretty-off',
                        action='store_true', default=False,
                        help='turn pretty print off')

    parser.add_argument('-l', '--lang',
                        action='store', default=None,
                        help='specify language, example: -l cpp11')

    parser.add_argument('--time-limit',
                        action='store', default=10,
                        help='specify execution time limit, example: --time-limit 2')

    parser.add_argument('-c', '--contest',
                        action='store', default=None,
                        help='specify contest when getting standings')

    parser.add_argument('--group',
                        action='store', default=None,
                        help='specify group')

    parser.add_argument('-w', '--watch',
                        action='store_true', default=False,
                        help='watch submission status')

    parser.add_argument('-v', '--verbose',
                        action='store_true', default=False,
                        help='show more when looking at standings')

    parser.add_argument('-a', '--all',
                        action='store_true', default=False,
                        help='show common standings')

    parser.add_argument('-t', '--top',
                        type=int, nargs='?', const=10, default=10,
                        help='number of top contestants to print')

    parser.add_argument('-s', '--sort',
                        choices=['solves', 'index', 'id'],
                        type=str, nargs='?', const='solves', default=None,
                        help='sort by: solves (default), index (id)')

    parser.add_argument('-g', '--guru',
                        action='store_true', default=False,
                        help='submit to acmsguru problemset')

    parser.add_argument('-n', '--number',
                        type=int, action='store', default=10,
                        help='number of tests')

    parser.add_argument('-r', '--reverse',
                        action='store_true', default=False,
                        help='Hacking with reversed submissions')

    args = parser.parse_args()

    # deal with short commands
    if args.command == 'st':
        args.command = 'standings'
    elif args.command == 'pb':
        args.command = 'problems'
    if args.sort == 'id':
        args.sort = 'index'

    # do stuff
    if args.command == 'gcon':
        # set group contest
        # check if bad input
        if args.group is None:
            group = input('Group Id: ')
        else:
            group = args.group

        if args.contest is None:
            contest = input('Contest number: ')
        else:
            contest = args.contest
        config['contest'] = contest
        config['group'] = group
        cf_utils.write_data_to_file(config, config_loc)
        print('Group set to ' + group)
        print('Contest set to ' + contest)

    elif args.command == 'gym' or args.command == 'con':
        # set contest
        # check if bad input
        if args.id is not None:
            contest = args.id
        else:
            contest = input('Contest/Gym number: ')
        config['contest'] = contest
        config['group'] = None
        cf_utils.write_data_to_file(config, config_loc)
        if len(contest) >= 6:
            print('Gym set to ' + contest)
        else:
            print('Contest set to ' + contest)
    elif args.command == 'groups':
        cf_groups.load_groups(args.pretty_off)
    elif args.command == 'gcontests':
        if args.group is None:
            group = input('Group Id: ')
        else:
            group = args.group
        cf_groups.load_contests(group, args.pretty_off)
    elif args.command == 'contests':
        cf_contests.load_contests(args.pretty_off)
    elif args.command == 'gyms':
        cf_gyms.load_gyms(args.pretty_off)
    elif args.command == 'ext':
        if len(args.option) > 1:
            print('Bad input')
            return
        if len(args.option) == 1:
            def_ext = args.option[0]
        else:
            def_ext = input('Default file extension: ')
        config['ext'] = def_ext
        cf_utils.write_data_to_file(config, config_loc)
        print('Default extension set to ' + def_ext)

    elif args.command == 'info':
        print('Handle: ' + str(config.get('handle', None)))
        print('Group ID: ' + str(config.get('group', None)))
        print('Contest ID: ' + str(config.get('contest', None)))

    elif args.command == 'login':
        # set login info
        if args.handle is None:
            cf_login.set_login()
        else:
            cf_login.set_login(args.handle)

    elif args.command == 'peek':
        # look at last submission
        cf_submit.peek(config.get('handle', None))

    elif args.command == 'watch':
        cf_submit.watch(config.get('handle', None))

    elif args.command == 'time':
        cf_standings.print_time(args.group or config.get('group', None),
                                args.contest or config.get('contest', None))

    elif args.command == 'standings':
        # look at standings
        cf_standings.print_standings(args.group or config.get('group', None),
                                     args.contest or config.get(
                                         'contest', None),
                                     args.verbose, args.top, args.sort, args.all)

    elif args.command == 'problems':
        # look at problem stats
        cf_problems.load_problems(args.pretty_off)

    elif args.command == 'submit':
        # open browser
        browser = cf_login.login()
        if browser is not None:
            cf_submit.submit_files(
                browser, config.get('handle', None),
                args.group or config.get('group', None),
                args.contest or config.get('contest', None),
                args.prob,
                config.get('ext', None),
                args.lang, args.option, args.watch, args.guru
            )
    elif args.command == 'test':
        if not os.path.isdir('files'):
            print('Please import problem test cases first by:\n\tcf parse -p a')
            return
        if len(args.option) == 0:
            print('Please select the source file')
            return
        cf_test.test(args.option[0], args.lang)
    elif args.command == 'parse':
        if args.prob is None:
            problem = input('Problem Id: ')
        else:
            problem = args.prob
        if len(problem) < 3:
            cf_parse.parse(config.get('group', None),
                           config.get('contest', None),
                           str(problem).upper())
        else:
            split = re.split(r'(\D+)', problem)
            cf_parse.parse(config.get('group', None), split[0], str(
                split[1] + split[2]).upper())
    elif args.command == 'open':
        if args.prob is None:
            print('Please select problem to open!!')
            return
        if config.get('group', None) is None:
            if len(args.prob) < 3:
                webbrowser.open('https://codeforces.com/contest/%s/problem/%s'
                                % (config.get('contest', None), str(args.prob).upper()), 2)
            else:
                split = re.split(r'(\D+)', args.prob)
                webbrowser.open('https://codeforces.com/contest/%s/problem/%s'
                                % (split[0], str(split[1] + split[2]).upper()), 2)
        else:
            if len(args.prob) < 3:
                webbrowser.open('https://codeforces.com/group/%s/contest/%s/problem/%s'
                                % (config.get('group', None), config.get('contest', None),
                                   str(args.prob).upper()), 2)
            else:
                split = re.split(r'(\D+)', args.prob)
                webbrowser.open('https://codeforces.com/group/%s/contest/%s/problem/%s'
                                % (config.get('contest', None), split[0],
                                   str(split[1] + split[2]).upper()), 2)
    elif args.command == 'hack':
        if 'standings' in args.option:
            if args.contest is None:
                contest = config.get('contest', None)
            else:
                contest = args.contest
            cf_hack.print_standings(contest, args.top, args.all)
            return
        if len(args.option) < 3:
            print(
                'Please select your <generator-source> [<tle-generator-source>] <checker-source> <answer-source> !!')
            return
        if args.prob is None:
            print('Please select problem to try!!')
            return

        if len(args.option) == 3:
            cf_hack.begin_hack(contest=config.get('contest', None), problem=args.prob, generator=args.option[0],
                               tle_generator=None, checker=args.option[1], correct_solution=args.option[2],
                               max_tests=args.number, reverse=args.reverse, time_limit=args.time_limit)
        else:
            cf_hack.begin_hack(contest=config.get('contest', None), problem=args.prob, generator=args.option[0],
                               tle_generator=args.option[1], checker=args.option[2], correct_solution=args.option[3],
                               max_tests=args.number, reverse=args.reverse, time_limit=args.time_limit)
    elif args.command == 'completion':
        os.system(
            'sudo ln -sf {}/bash_completion/cf /etc/bash_completion.d/cf'.format(
                os.path.dirname(os.path.abspath(__file__))))
        os.system('source /etc/bash_completion.d/cf')
    elif args.command == 'version':
        print('cf-submit %s' % __version__)
    else:
        print('UNKNOWN COMMAND')



import glob
import os
from subprocess import Popen

from .cf_colors import Colors


def test(source, lang):
    info = source.split('.')
    if lang is None:
        lang = info[-1]
    if lang == 'py':
        print('%sPlease specify the version of python : py2 or py3%s' % (Colors.FAIL, Colors.END))
        return
    print('%sCompiling your %s solution...' % (Colors.OK_BLUE, lang))
    comp(source, lang, info[0])

    print('Executing...%s' % Colors.END)
    i = 1
    success = 0
    failed = 0
    for file_name in glob.iglob('files/test*.in'):
        with open(file_name, 'r') as input_file:
            with open(file_name.replace('.in', '.out'), 'w') as output_file:
                print('%s#################### TEST %d ####################%s\n' % (Colors.WARNING, i, Colors.END))
                i += 1
                execute(source, lang, info[0], input_file, output_file)
                process = Popen(['%s/bin/cf_checker' % os.path.dirname(os.path.abspath(__file__)),
                                 file_name, file_name.replace('.in', '.out'), file_name.replace('.in', '.ans')])
                process.wait(timeout=5)
                exit_code = process.returncode
                print('\n%sInput:%s' % (Colors.OK_BLUE, Colors.END))
                print(open(file_name, 'r').read(), '\n')
                if exit_code == 0:
                    success += 1
                else:
                    print('%sExpected:%s' % (Colors.OK_BLUE, Colors.END))
                    print(
                        str(open(file_name.replace('.in', '.ans'), 'r').read()).strip(), '\n')
                    failed += 1
                print('%sOutput:%s' % (Colors.OK_BLUE, Colors.END))
                print(
                    str(open(file_name.replace('.in', '.out'), 'r').read()).strip(), '\n')

    print('%s################################################%s\n' % (Colors.WARNING, Colors.END))
    print('%sSuccess: %d, %sFailed: %d%s' % (Colors.OK_GREEN, success, Colors.FAIL, failed, Colors.END))


def execute(source, lang, executable, input_file, output_file):
    if lang == 'cpp' or lang == 'c':
        cmd = './%s' % executable
    elif lang == 'java':
        cmd = 'java -DLOCAL %s' % executable
    elif lang == 'kt':
        cmd = 'java -DLOCAL -jar %s.jar' % executable
    elif lang == 'py2':
        cmd = 'python2 %s ' % source
    elif lang == 'py3':
        cmd = 'python3 %s ' % source
    else:
        print('Sorry language not supported!')
        return exit(-1)
    Popen(cmd, stdin=input_file, stdout=output_file, shell=True).wait()


def comp(source, lang, executable):
    if lang == 'cpp':
        cmd = 'g++ %s -DLOCAL -O2 -o %s' % (source, executable)
    elif lang == 'c':
        cmd = 'gcc %s -DLOCAL -O2 -o %s' % (source, executable)
    elif lang == 'java':
        cmd = 'javac %s' % source
    elif lang == 'kt':
        cmd = 'javac %s -include-runtime -d %s' % (source, executable)
    else:
        cmd = []
    Popen(cmd, shell=True).wait()
